function [M S] = locMassStiffH(v,lgorder,type)
%
% Make the mass and stiffness matrices
% for quadratic Lagrange functions in a Hexagon.
%
% Inputs:
% v    : [x1 y1; x2 y2; x3 y4; x4 y4], the 4 vertices of the quad P, ccw
% type : type of barycentric coordinate:
%        type=1 => Wachspress
%        type=2 => MV local
%        type=3 => bilinear, if n=4.
%        type=4 => family
%
% Outputs:
% M = (m_{ij}), i,j=1,,,.8. m_{ij} = \int_P L_i L_j.
% S = (s_{ij}), i,j=1,,,.8. s_{ij} = \int_P grad(L_i) grad(L_j),
% where
% (L_1,...,L_8) = (L_{11},L_{12},L_{22},...,L_{41}).
% 
% Author: Michael Floater and Ming-Jun Lai
% e-mail: michaelf@ifi.uio.no and mjlai@math.uga.edu
% Created: Nov 2014, using Matlab R2014a
% Copyright 2014

M = zeros(12,12);
S = M;
v1=v(1,:);v2=v(2,:);v3=v(3,:);v4=v(4,:);v5=v(5,:);v6=v(6,:);
%d1=det([v2-v1;v6-v1]); d2=det([v2-v1;v3-v2]);
%d3=det([v3-v6;v3-v2]); d4=det([v3-v6;v6-v1]);
%d5=det([v4-v3;v6-v3]); d6=det([v4-v3;v5-v4]);
%d7=det([v5-v6;v5-v4]); d8=det([v5-v6;v6-v3]);
d1=(v2(1,1)-v1(1,1))*(v6(1,2)-v1(1,2))-(v2(1,2)-v1(1,2))*(v6(1,1)-v1(1,1));
%d1=det([v2-v1;v6-v1]); 
d2=(v2(1,1)-v1(1,1))*(v3(1,2)-v2(1,2))-(v2(1,2)-v1(1,2))*(v3(1,1)-v2(1,1));
%d2=det([v2-v1;v3-v2]);
d3=(v3(1,1)-v6(1,1))*(v3(1,2)-v2(1,2))-(v3(1,2)-v6(1,2))*(v3(1,1)-v2(1,1));
%d3=det([v3-v6;v3-v2]);
d4=(v3(1,1)-v6(1,1))*(v6(1,2)-v1(1,2))-(v3(1,2)-v6(1,2))*(v6(1,1)-v1(1,1));
%d4=det([v3-v6;v6-v1]);
d5=(v4(1,1)-v3(1,1))*(v6(1,2)-v3(1,2))-(v4(1,2)-v3(1,2))*(v6(1,1)-v3(1,1));
%d5=det([v4-v3;v6-v3]); 
d6=(v4(1,1)-v3(1,1))*(v5(1,2)-v4(1,2))-(v4(1,2)-v3(1,2))*(v5(1,1)-v4(1,1));
%d6=det([v4-v3;v5-v4]);
d7=(v5(1,1)-v6(1,1))*(v5(1,2)-v4(1,2))-(v5(1,2)-v6(1,2))*(v5(1,1)-v4(1,1));
%d7=det([v5-v6;v5-v4]); 
d8=(v5(1,1)-v6(1,1))*(v6(1,2)-v3(1,2))-(v5(1,2)-v6(1,2))*(v6(1,1)-v3(1,1));
%d8=det([v5-v6;v6-v3]);

% n-point Gauss quadrature on [0,1].
% for n=2 or n=3.
n=lgorder;
[g0,w0]=lg_rule(n);
g = 0.5 + g0 * 0.5;
w = 0.5 * w0;
%f1 = zeros(10,1);
%f2 = zeros(10,1);
for i=1:n
  for j=1:n
    s=g(i);
    t=g(j);
    x = (1-s)*(1-t)*v1 + s*(1-t)*v2 + s*t*v3 + (1-s)*t*v6;
    [L dL] = Lag2dDeg2(v,x,type);
    Mloc = L * L';
    Sloc = dL * dL';
    d = (1-s)*(1-t)*d1 + s*(1-t)*d2 + s*t*d3 + (1-s)*t*d4;
    wd = w(i)*w(j)*d;
    M = M + wd * Mloc;
    S = S + wd * Sloc;
  end
end

for i=1:n
  for j=1:n
    s=g(i);
    t=g(j);
    x = (1-s)*(1-t)*v3 + s*(1-t)*v4 + s*t*v5 + (1-s)*t*v6;
    [L dL] = Lag2dDeg2(v,x,type);
    Mloc = L * L';
    Sloc = dL * dL';
    d = (1-s)*(1-t)*d5 + s*(1-t)*d6 + s*t*d7 + (1-s)*t*d8;
    wd = w(i)*w(j)*d;
    M = M + wd * Mloc;
    S = S + wd * Sloc;
  end
end