function [M S] = locMassStiffT(v,lgorder,type)
%
% Make the mass and stiffness matrices
% for quadratic Lagrange functions in a quad.
%
% Inputs:
% v    : [x1 y1; x2 y2; x3 y4; x4 y4], the 4 vertices of the quad P, ccw
% type : type of barycentric coordinate:
%        type=1 => Wachspress
%        type=2 => MV local
%        type=3 => bilinear, if n=4.
%        type=4 => family
%
% Outputs:
% M = (m_{ij}), i,j=1,,,.8. m_{ij} = \int_P L_i L_j.
% S = (s_{ij}), i,j=1,,,.8. s_{ij} = \int_P grad(L_i) grad(L_j),
% where
% (L_1,...,L_8) = (L_{11},L_{12},L_{22},...,L_{41}).
% 
% Author: Michael Floater and Ming-Jun Lai
% e-mail: michaelf@ifi.uio.no and mjlai@math.uga.edu
% Created: Nov 2014, using Matlab R2014a
% Copyright 2014

M = zeros(6,6);
S = zeros(6,6);
V1=v;
v=[v;(v(1,:)+v(3,:))/2];
d1 = det([v(2,:)-v(1,:);v(4,:)-v(1,:)]);
d2 = det([v(2,:)-v(1,:);v(3,:)-v(2,:)]);
d3 = det([v(3,:)-v(4,:);v(3,:)-v(2,:)]);
d4 = det([v(3,:)-v(4,:);v(4,:)-v(1,:)]);

% n-point Gauss quadrature on [0,1].
% for n=2 or n=3.
n=lgorder;
[g0,w0]=lg_rule(n);
g = 0.5 + g0 * 0.5;
w = 0.5 * w0;


f1 = zeros(6,1);
f2 = zeros(6,1);
for i=1:n
  for j=1:n
    s=g(i);
    t=g(j);
    x = (1-s)*(1-t)*v(1,:) + s*(1-t)*v(2,:) + s*t*v(3,:) + (1-s)*t*v(4,:);
    [L dL] = Lag2dDeg2(V1,x,type);
    Mloc = L * L';
    Sloc = dL * dL';
    d = (1-s)*(1-t)*d1 + s*(1-t)*d2 + s*t*d3 + (1-s)*t*d4;
    wd = w(i)*w(j)*d;
    M = M + wd * Mloc;
    S = S + wd * Sloc;
  end
end

